C**********************************************************************
C                           ADAPT                                     *
C                         Version 5                                   *
C**********************************************************************
C
C SOLUTION AND IMPLEMENTATION OF DISTRIBUTED LIFESPAN MODELS
C
C Gilbert Koch(*) and Johannes Schropp
C
C (*) Department of Pharmaceutical Sciences, State University of
C     New York at Buffalo, 403 Kapoor Hall, Buffalo, New York 14214, USA
C     Tel.: +1-716-6454816
C     Fax: +1-716-8296569
C     E-mail: gilbertk@buffalo.edu
C
C
C Example 1: PKPD test problem (Version: July 22nd, 2013)
C
C
C**********************************************************************

C######################################################################C

        Subroutine SYMBOL
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

CC
C----------------------------------------------------------------------C
C                   Enter as Indicated                                 C
C----c-----------------------------------------------------------------C

      NDEqs   =  0  ! Enter # of Diff. Eqs.
      NSParam =  5  ! Enter # of System Parameters.
      NVparam =  1  ! Enter # of Variance Parameters.
      NSecPar =  2  ! Enter # of Secondary Parameters.
      NSecOut =  0  ! Enter # of Secondary Outputs (not used).
      Ieqsol  =  3  ! Model type: 1 - DIFFEQ, 2 - AMAT, 3 - OUTPUT only.
      Descr   = 'DLSM'
	
CC
C----------------------------------------------------------------------C
C     Enter Symbol for Each System Parameter (eg. Psym(1)='Kel')       C
C----c-----------------------------------------------------------------C

        Psym(1) = 'Smax'
        Psym(2) = 'SC50'
        Psym(3) = 'kin0'
        Psym(4) = 'a'
        Psym(5) = 'b'

CC
C----------------------------------------------------------------------C
C    Enter Symbol for Each Variance Parameter {eg: PVsym(1)='Sigma'}   C
C----c-----------------------------------------------------------------C



CC
C----------------------------------------------------------------------C
C    Enter Symbol for Each Secondary Parameter {eg: PSsym(1)='CLt'}    C
C----c-----------------------------------------------------------------C

        PSsym(1) = 'Mean'
	PSsym(2) = 'Var'

C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C
C   Additional Functions and Subroutines
C######################################################################C


C--- Multiple Dosing

	REAL*8 FUNCTION calc_c(t,con)

	real*8 t,td,c,ka,ke,V,dose,con(5),ndose

	integer i,j

C	PK parameter
	ka = 0.7
	ke = 0.5
	V = 1

	if (t .GE. 0) then
	   ndose = con(1)
	   j = 1
	   do i=1,ndose
	      j = j + 2
              if (t .le. con(j)) then 
                 go to 100
              end if
           end do
100	   ndose = i-1
	   c  = 0
           j = 1
           do i=1,ndose
              j = j + 2
              td = t - con(j)
	      dose = con(j-1)
              c  = c +((ka*dose)/((ka-ke)*V))*(exp(-ke*td)-exp(-ka*td));
           end do
	else
	   c = 0
	end if
	calc_c = c
	return
	END


C--- LogGamma Function

	REAL*8 FUNCTION gammln(xx)
	real*8 xx
	integer j
	real*8 ser,stp,tmp,x,y,cof(6)
	data cof,stp/76.18009172947146d0,-86.50532032941677d0,
     * 24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     * -.5395239384953d-5,2.5066282746310005d0/

	x=xx
	y=x
	tmp=x+5.5d0
	tmp=(x+0.5d0)*dlog(tmp)-tmp
	ser=1.000000000190015d0
	do j=1,6
	  y=y+1.d0
	  ser=ser+cof(j)/y
	end do
	gammln=tmp+dlog(stp*ser/x)

	return
	END



C--- CDF Weibull

	REAL*8 FUNCTION CDF_WBD(s)
	implicit none
	include 'globals.inc'
	include 'model.inc'
	real*8 s

        CDF_WBD = 1-dexp(-(s/P(5))**P(4))

	return
	END


C--- kin(t)

	REAL*8 FUNCTION fun_kin(s,con)
	implicit none
	include 'globals.inc'
	include 'model.inc'
 	real*8 s,c,calc_c,con(5)
	
	if (s > 0) then
	   c = calc_c(s,con)
	   fun_kin = P(3)*(1+(P(1)*c)/(P(2)+c))
	else
           fun_kin = P(3)
        end if

	return
	END	


C--- Integrand

	REAL*8 FUNCTION fun_f(t,con,s)
	implicit none
 	Include 'globals.inc'
       	Include 'model.inc'
	real*8 s,t,con(5),fun_kin,CDF_WBD

        fun_f = (1-CDF_WBD(s))*fun_kin(t-s,con)
      
	return
	END


C--- Integrator - Riemann sum

	REAL*8 FUNCTION fun_Riem(nalpha,delta,con,t)
	implicit none
 	Include 'globals.inc'
       	Include 'model.inc'
	real*8 delta,t,con(5),sum,fun_f
	integer i,nalpha

	sum=0
	do i=1,nalpha
	   sum = sum + fun_f(t,con,i*delta)
	end do
        fun_Riem = delta*sum
	
	return
	END
 

C--- Integrator - Trapezodial rule

	REAL*8 FUNCTION fun_Trap(nalpha,delta,con,t)
	implicit none
 	include 'globals.inc'
       	include 'model.inc'
	integer i,nalpha
	real*8 delta,t,con(5),sum,fun_f

	sum = 0
 	do i=1,nalpha
	   sum = sum + fun_f(t,con,i*delta) + fun_f(t,con,(i-1)*delta)
	end do
        fun_Trap = (delta/2)*sum
	
	return
	END


C######################################################################C

        Subroutine DIFFEQ(T,X,XP)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Real*8 T,X(MaxNDE),XP(MaxNDE)
CC
C----------------------------------------------------------------------C
C     Enter Differential Equations Below  {e.g.  XP(1) = -P(1)*X(1) }  C
C----c-----------------------------------------------------------------C

C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End


C######################################################################C

        Subroutine OUTPUT(Y,T,X)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Real*8 Y(MaxNOE),T,X(MaxNDE)

C----------------------------------------------------------------------C
C     Enter Output Equations Below   {e.g.  Y(1) = X(1)/P(2) }         C
C----c-----------------------------------------------------------------C

        Real*8 delta,alpha,mean
	Real*8 gammln,fun_Trap,fun_Riem,CDF_WBD
	Real*8 con_1(5),con_2(5),con_3(5),con_4(5)
	Integer nalpha,n

C Dosing parameter
	DATA con_1/2,10,0,10,7/
	DATA con_2/2,50,0,50,7/
	DATA con_3/2,100,0,100,7/
	DATA con_4/2,500,0,500,7/
        
C Partition and alpha
	n = 500
        alpha = 0.01

C Calculate mean
	mean = P(5)*dexp(gammln(1+1/P(4)))

C Calculate delta
	delta = mean/n

C Calculate nalpha
        nalpha = n
	do while (1-CDF_WBD(nalpha*delta) > alpha)
	  nalpha = nalpha+1
        end do

C Calculate Riemann sum
C        Y(1) = fun_Riem(nalpha,delta,con_1,t)
C        Y(2) = fun_Riem(nalpha,delta,con_2,t)
C        Y(3) = fun_Riem(nalpha,delta,con_3,t)
C        Y(4) = fun_Riem(nalpha,delta,con_4,t)

C Calculate Trapezodial rule
        Y(1) = fun_Trap(nalpha,delta,con_1,t)
        Y(2) = fun_Trap(nalpha,delta,con_2,t)
        Y(3) = fun_Trap(nalpha,delta,con_3,t)
        Y(4) = fun_Trap(nalpha,delta,con_4,t)      


C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine VARMOD(V,T,X,Y)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Real*8 V(MaxNOE),T,X(MaxNDE),Y(MaxNOE)

CC
C----------------------------------------------------------------------C
C       Enter Variance Model Equations Below                           C
C        {e.g. V(1) = (PV(1) + PV(2)*Y(1))**2 }                        C
C----c-----------------------------------------------------------------C

        V(1) = (PV(1)*Y(1))**2
        V(2) = (PV(1)*Y(2))**2
        V(3) = (PV(1)*Y(3))**2
        V(4) = (PV(1)*Y(4))**2
 

C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine COVMOD(Pmean, ICmean, PC)
C  Defines any covariate model equations (MLEM, ITS)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Real*8 PC(MaxNCP)
        Real*8 Pmean(MaxNSP+MaxNDE), ICmean(MaxNDE)

CC
C----------------------------------------------------------------------C
C     Enter # of Covariate Parameters                                  C
C----c-----------------------------------------------------------------C

        NCparam = 0    ! Enter # of Covariate Parameters.

CC
C----------------------------------------------------------------------C
C   Enter Symbol for Covariate Params {eg: PCsym(1)='CLRenal'}         C
C----c-----------------------------------------------------------------C


CC
C----------------------------------------------------------------------C
C   For the Model Params. that Depend on Covariates Enter the Equation C
C         {e.g. Pmean(1) =  PC(1)*R(2) }                               C
C----c-----------------------------------------------------------------C


C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine POPINIT(PmeanI,ICmeanI,PcovI,ICcovI, PCI)
C  Initial parameter values for population program parameters (ITS, MLEM)

        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Integer I,J
        Real*8 PmeanI(MaxNSP+MaxNDE), ICmeanI(MaxNDE)
        Real*8 PcovI(MaxNSP+MaxNDE,MaxNSP+MaxNDE), ICcovI(MaxNDE,MaxNDE)
        Real*8 PCI(MaxNCP)

CC
C----------------------------------------------------------------------C
C  Enter Initial Values for Population Means                           C
C          {  e.g. PmeanI(1) = 10.0    }                               C
C----c-----------------------------------------------------------------C


CC
C----------------------------------------------------------------------C
C   Enter Initial Values for Pop. Covariance Matrix (Lower Triang.)    C
C         {  e.g. PcovI(2,1) = 0.25    }                               C
C----c-----------------------------------------------------------------C


CC
C----------------------------------------------------------------------C
C   Enter Values for Covariate Model Parameters                        C
C         {  e.g. PCI(1) = 2.0    }                                    C
C----c-----------------------------------------------------------------C


C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine PRIOR(Pmean,Pcov,ICmean,ICcov)
C  Parameter mean and covariance values for MAP estimation (ID,NPD,STS)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Integer I,J
        Real*8 Pmean(MaxNSP+MaxNDE), ICmean(MaxNDE)
        Real*8 Pcov(MaxNSP+MaxNDE,MaxNSP+MaxNDE), ICcov(MaxNDE,MaxNDE)

CC
C----------------------------------------------------------------------C
C  Enter Nonzero Elements of Prior Mean Vector                         C
C          {  e.g. Pmean(1) = 10.0    }                                C
C----c-----------------------------------------------------------------C


CC
C----------------------------------------------------------------------C
C   Enter Nonzero Elements of Covariance Matrix (Lower Triang.)       C
C         {  e.g. Pcov(2,1) = 0.25    }                                C
C----c-----------------------------------------------------------------C


C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine SPARAM(PS,P,IC)
        Implicit None

        Include 'globals.inc'

        Real*8 PS(MaxNSECP), P(MaxNSP+MaxNDE), IC(MaxNDE) 
	Real*8 gammln
CC
C----------------------------------------------------------------------C
C       Enter Equations Defining Secondary Paramters                   C
C           {  e.g.  PS(1) = P(1)*P(2)   }                             C
C----c-----------------------------------------------------------------C

C  Calcualte mean and variance 

 	PS(1) = P(5)*dexp(gammln(1+1/P(4)))
	PS(2) = (P(5)**2)*dexp(gammln(1+2/P(4)))-PS(1)**2
	


        
C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End
        
C######################################################################C

        Subroutine AMAT(A)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Integer I,J
        Real*8 A(MaxNDE,MaxNDE)

        DO I=1,Ndeqs
           Do J=1,Ndeqs
              A(I,J)=0.0D0
           End Do
        End Do

CC
C----------------------------------------------------------------------C
C    Enter non zero elements of state matrix  {e.g.  A(1,1) = -P(1) }  C
C----c-----------------------------------------------------------------C



C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C
