C**********************************************************************
C                            ADAPT                                    *
C                          Version 5                                  *
C**********************************************************************

C**********************************************************************
C                                                                     *
C  Copyright (c) 1988-2009 by                                         *
C  D. Z. D'Argenio, Biomedical Simulations Resource  (BMSR)           *
C  University of Southern California                                  *
C  Los Angeles, California 90089-1111                                 *
C  All rights reserved.                                               *
C                                                                     *
C  The ADAPT software is furnished by the Biomedical Simulations      *
C  Resource under the terms of a release agreement.  ADAPT may be     *
C  used or copied only under the terms of the release agreement.      *
C  No title to or ownership of the software is hereby transferred.    *
C                                                                     *
C**********************************************************************


C**********************************************************************
C                         GLOBALS.INC                                 *
C**********************************************************************

C**********************************************************************
C                                                                     *
C Contains matrix dimension values, associated variable declarations, *
C tolerance values, and option flags for several of the numerical     *
C methods, and other set-up variables required by the ADAPT programs  *
C and subroutines.                                                    *
C                                                                     *
C If this file is modified then RecompileADAPT must be run            *
C                                                                     *
C**********************************************************************


C  Define maximum value constants.
      Integer MaxNDE,MaxNSP,MaxNRI,MaxNBI,MaxND
      Integer MaxNOE,MaxNOB,MaxOBE
      Integer MaxNVP,MaxSIM,MaxPLT,MaxNSECP,MaxNSO
      Integer MaxNSubs, MaxNITS, MaxNSAMP
      Integer MaxNPest, MaxNTest
      Integer MaxNCP
      Parameter (MaxNDE=25)    ! Max Number of Differential Equations
      Parameter (MaxNSP=35)    ! Max Number of System Parameters
      Parameter (MaxNRI=10)    ! Max Number of Rate Inputs
      Parameter (MaxNBI=10)    ! Max Number of Bolus Inputs
      Parameter (MaxND=200)    ! Max Number of Dose Events
      Parameter (MaxNOE=15)    ! Max Number of Output Equations
      Parameter (MaxNOB=200)   ! Max Number of Observations per Output
      Parameter (MaxOBE=300)   ! Max Number of Total Observations
      Parameter (MaxNVP=20)    ! Max Number of Variance Parameters
      Parameter (MaxSIM=5000)  ! Max Number of Simulations
      Parameter (MaxNSECP=15)  ! Max Number of Secondary Parameters
      Parameter (MaxNSO=10)    ! Max Number of Secondary Outputs
      Parameter (MaxPLT=1000)  ! Number of Points used for Smooth Plots
                               !   MaxPLT>=MaxOBE
      Parameter (MaxNSubs=500) ! Max Number of Subjects
      Parameter (MaxNITS=1000) ! Max Number of ITS and EM Iterations
      Parameter (MaxNSAMP=3000)! Max Number of Samples per EM Iteration
                               !   for Importance Sampler
      Parameter (MaxNPest=25)  ! Max Number of Parameters Estimated 
                               !   ( <= MaxNSP+MaxNDE )
      Parameter (MaxNTest=20)  ! Max Number of Sample Times Optimized 
                               !   ( <= MaxNOB )
      Parameter (MaxNCP = 35)  ! Max Number of Covariate Model Parameters

C  Stopping tolerance for Nelder-Mead algorithm.
      Real*8 Reqmin
      Parameter (Reqmin = 1.0d-08)

C  Missing data number.
      Real*8 Misdat
      Parameter (Misdat=-1.0D0)

C  Define constants for LSODA (see documentation in LSODA code).
C   Note that DimRW must be set MANUALLY based on the value of MaxNDE.
      Integer ITOL,ITASK,IOPT,DimRW,LIW,JT
      Real*8 RTOL,ATOL
      Parameter (ITOL=1)
      Parameter (ITASK=1)
      Parameter (IOPT=0)
      Parameter (LIW=MaxNDE+20)
      Parameter (JT=2)
      Parameter (DimRW=872)     ! DimRW = 22 + MaxNDE*Max(16,MaxNDE+9)
      Parameter (RTOL=1.0D-06)  ! Rel. error tolerance for LSODA
      Parameter (ATOL=1.0D-06)  ! Abs. error tolerance for LSODA

C  Default diplay type for Sun and Linux versions
       Integer Term
       Parameter (Term = 1)  ! Term = 1 for VT/TEK Terminal or Emulation  
C                            ! Term = 2 for XWindow display 
 
 
C  ADAPT Directory Full Path - 260 is the maximum path length
      Character*260 AdaptDirPath
      Parameter (AdaptDirPath='C:\ADAPT 5\')
