C**********************************************************************
C                           ADAPT                                     *
C                         Version 5                                   *
C**********************************************************************
C
C SOLUTION AND IMPLEMENTATION OF DISTRIBUTED LIFESPAN MODELS
C
C Gilbert Koch(*) and Johannes Schropp
C
C (*) Department of Pharmaceutical Sciences, State University of
C     New York at Buffalo, 403 Kapoor Hall, Buffalo, New York 14214, USA
C     Tel.: +1-716-6454816
C     Fax: +1-716-8296569
C     E-mail: gilbertk@buffalo.edu
C
C
C Example 2: Hemoglobin (Version: July 22nd, 2013)
C
C
C**********************************************************************
C**********************************************************************

C######################################################################C

        Subroutine SYMBOL
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

CC
C----------------------------------------------------------------------C
C                   Enter as Indicated                                 C
C----c-----------------------------------------------------------------C

      NDEqs   =  0  ! Enter # of Diff. Eqs.
      NSParam =  7  ! Enter # of System Parameters.
      NVparam =  1  ! Enter # of Variance Parameters.
      NSecPar =  4  ! Enter # of Secondary Parameters.
      NSecOut =  0  ! Enter # of Secondary Outputs (not used).
      Ieqsol  =  3  ! Model type: 1 - DIFFEQ, 2 - AMAT, 3 - OUTPUT only.
      Descr   = 'Precursor DLSM for Hgb Data'
	
CC
C----------------------------------------------------------------------C
C     Enter Symbol for Each System Parameter (eg. Psym(1)='Kel')       C
C----c-----------------------------------------------------------------C

        Psym(1) = 'Vmax'
        Psym(2) = 'Km'
        Psym(3) = 'Hen'
        Psym(4) = 'Smax'
	Psym(5) = 'SC50'
	Psym(6) = 'TP'
	Psym(7) = 'b'
CC
C----------------------------------------------------------------------C
C    Enter Symbol for Each Variance Parameter {eg: PVsym(1)='Sigma'}   C
C----c-----------------------------------------------------------------C



CC
C----------------------------------------------------------------------C
C    Enter Symbol for Each Secondary Parameter {eg: PSsym(1)='CLt'}    C
C----c-----------------------------------------------------------------C

        PSsym(1) = 'Mean_TR'
	PSsym(2) = 'Var_TR'
	PSsym(3) = 'Een'
	PSsym(4) = 'R0'

C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C
C   Additional Functions and Subroutines
C######################################################################C


C----------------------------------------------------------------------C
C   PK        
C----------------------------------------------------------------------C

	REAL*8 FUNCTION calc_kin(t)
	implicit none
	include 'globals.inc'
	include 'model.inc'
	real*8 t,td,c,coef1
	real*8 ke,Een,kin,KH
	real*8 dose,con_t(187),con_d(186),con_l(186)
	real*8 ndose
	real*8 Vmax,Km,Hen,Smax,SC50,TP,bb
	real*8 tmp1,tmp2,tmp3,tmp4,tmp5
	integer i,j

	data con_t/  0,  2,  4,  7,  9, 11, 16, 18, 21, 23, 25, 28, 30, 
     * 32, 35, 37, 39, 42, 49, 51, 53, 56, 58, 63, 65, 67, 70, 72, 74, 
     * 77, 79, 81, 84, 86, 88, 91, 93, 95, 98,100,102,105,107,109,112,
     *114,116,119,121,123,126,128,130,133,135,137,140,142,144,147,149,
     *151,154,158,161,163,165,168,170,172,175,177,179,182,184,186,189,
     *191,193,196,200,203,207,210,212,217,219,224,228,230,232,235,238,
     *240,242,245,247,252,254,256,259,261,263,265,268,270,272,275,277,
     *280,282,284,287,289,291,294,296,298,301,303,305,308,310,312,315,
     *317,319,322,324,326,329,331,333,336,338,340,343,345,347,350,352,
     *354,357,359,361,364,366,368,371,373,375,378,380,382,385,387,389,
     *392,394,396,401,403,406,408,410,413,415,417,420,422,424,427,429,
     *431,434,436,438,441,443,445,448,450,452,455,457,459,461/

	data con_d/1100,1100,1100,1100,1100,1100,1100,1100,1100,1100,
     *1100,1100,1100,1100,1100,1100,1100,1100,1100,1100,1100,1100,1100,
     *1100,1100,1100,1100,1100,1100,1100,1100,1100,1100,1100,1100, 900, 
     * 900, 900, 900, 900, 900, 900, 900, 900, 900, 900, 900, 900, 900, 
     * 900, 900, 900, 900, 900, 900, 900, 700, 700, 700, 700, 700, 700, 
     * 700, 700, 700, 700, 700, 700, 700, 700, 700, 700, 700, 700, 500, 
     * 500, 500, 500, 500, 500, 500, 400, 400, 400, 400, 400, 400, 400, 
     * 400, 400, 400, 400, 400, 400, 400, 400, 400, 400, 400, 400, 400, 
     * 400, 400, 400, 400, 400, 400, 400, 400, 400, 400, 400, 400, 400, 
     * 400, 400, 400, 400,2000,2000,2000,2000,2000,2000,2000,2000,2000,
     *2000,2000,2000,2000,2000,2000,2000,2000,2000,2000,2000,2000,2000,
     *2000,2000,2000,2000,2000,2000,2000,2000,2000,2000,2000,2000,2000,
     *2000,2000,2000,2000,2000,2000,2000,1000,1000,1000,1000,1000,1000,
     *1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,
     *1000,1000,1000,1000,1000,1000,1000/


C-- Calculate length of intervals
	do i=1,186
	   con_l(i) = con_t(i+1) - con_t(i)
	end do


        Vmax = P(1)
        Km = P(2)
        Hen = P(3)
        Smax = P(4)
	SC50 = P(5)
	TP = P(6)
	bb = P(7)

	kin = 0D0

	KH = 29.5D0

	Een = (SC50*Hen)/(2*(1/bb)*KH*Smax-Hen)

	if (t .ge. 0) then
	   do i=1,186	      
	      if (t .ge. con_t(i) .and. t .lt. con_t(i+1)) then
		  tmp1 = (Smax/(con_l(i)*Vmax))
		  tmp2 = Km-SC50-(Km*Een)/(SC50+Een)

		  if ((SC50+Een+con_d(i))/(SC50+Een) .le. 0d0) then
             write(*,*) SC50,Een,con_d(i),(SC50+Een+con_d(i))/(SC50+Een)
	          end if

		  tmp3 = log((SC50+Een+con_d(i))/(SC50+Een))
		  tmp4 = (1-Een/(SC50+Een))*con_d(i)
		  tmp5 = (Smax*Een)/(SC50+Een)
		  kin = tmp1*(tmp2*tmp3+tmp4)+tmp5
	          go to 100  
	      end if
	   end do
	else 
	   kin = (Smax*Een)/(SC50+Een)
	end if

100	calc_kin = kin

	return
	END



C----------------------------------------------------------------------C
C   CDF Gamma 
C----------------------------------------------------------------------C

	REAL*8 FUNCTION cdf_GAM(tau,a,b)

	real*8 tau,a,b

	if (tau .ge. 0) then
	   cdf_GAM = 1d0-(1d0+b*tau)*exp(-tau*b)
 	else
	   cdf_GAM = 0
        end if

	return
	END


C----------------------------------------------------------------------C
C   Mean Gamma 
C----------------------------------------------------------------------C

	REAL*8 FUNCTION mean_GAM(a,b)

	real*8 a,b

	mean_GAM = a/b

	return
	END


C----------------------------------------------------------------------C
C   Var Gamma 
C----------------------------------------------------------------------C

	REAL*8 FUNCTION var_GAM(a,b)

	real*8 a,b

	var_GAM = a/b**2D0

	return
	END



C---------------------------------------------------------------------C
C   Integrand
C---------------------------------------------------------------------C

	REAL*8 FUNCTION fun_f(t,s)

	implicit none
 	include 'globals.inc'
       	include 'model.inc'
	real*8 s,calc_kin,LN,cdf_GAM,t
	integer i

        LN = cdf_GAM(s,2D0,P(7))
        fun_f = (1-LN)*calc_kin(t-s-P(6))
      
	return
	END


 
C---------------------------------------------------------------------C
C--- Integrator - Trapezodial rule
C---------------------------------------------------------------------C

	REAL*8 FUNCTION fun_Trap(nalpha,delta,t)
	implicit none
 	include 'globals.inc'
       	include 'model.inc'
	integer i,nalpha
	real*8 delta,t,sum,fun_f

	sum = 0
	do i=1,nalpha
	   sum = sum + fun_f(t,i*delta) + fun_f(t,(i-1)*delta)
	end do
	fun_Trap = (delta/2)*sum
	
	return
	END




C######################################################################C

        Subroutine DIFFEQ(T,X,XP)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Real*8 T,X(MaxNDE),XP(MaxNDE)
CC
C----------------------------------------------------------------------C
C     Enter Differential Equations Below  {e.g.  XP(1) = -P(1)*X(1) }  C
C----c-----------------------------------------------------------------C

C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End


C######################################################################C

        Subroutine OUTPUT(Y,T,X)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Real*8 Y(MaxNOE),T,X(MaxNDE)

C----------------------------------------------------------------------C
C     Enter Output Equations Below   {e.g.  Y(1) = X(1)/P(2) }         C
C----c-----------------------------------------------------------------C

	real*8 dose,V,ka,ke,c
        real*8 Tmax,delta,sum,LN,kin
        real*8 alpha,z,gamma,mean
	real*8 KH,tmp
	real*8 gammln,calc_c,fun_Riem,fun_Trap,mean_LND
	real*8 cdf_GAM,mean_GAM
	integer i,nalpha,idx,n
        
	KH = 29.5D0

C Partition and alpha
	n = 500
        alpha = 0.01D0

C Calculate delta
	delta = mean_GAM(2D0,P(7))/n

C Calculate nalpha 
        nalpha = n
	do while (1-cdf_GAM(nalpha*delta,2D0,P(7)) > alpha)
	  nalpha = nalpha+1
        end do

C Calculate Trapezoidal Rule

	Y(1) = KH*fun_Trap(nalpha,delta,t)



C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine VARMOD(V,T,X,Y)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Real*8 V(MaxNOE),T,X(MaxNDE),Y(MaxNOE)

CC
C----------------------------------------------------------------------C
C       Enter Variance Model Equations Below                           C
C        {e.g. V(1) = (PV(1) + PV(2)*Y(1))**2 }                        C
C----c-----------------------------------------------------------------C

        V(1) = (PV(1)*Y(1))**2

 

C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine COVMOD(Pmean, ICmean, PC)
C  Defines any covariate model equations (MLEM, ITS)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Real*8 PC(MaxNCP)
        Real*8 Pmean(MaxNSP+MaxNDE), ICmean(MaxNDE)

CC
C----------------------------------------------------------------------C
C     Enter # of Covariate Parameters                                  C
C----c-----------------------------------------------------------------C

        NCparam = 0    ! Enter # of Covariate Parameters.

CC
C----------------------------------------------------------------------C
C   Enter Symbol for Covariate Params {eg: PCsym(1)='CLRenal'}         C
C----c-----------------------------------------------------------------C


CC
C----------------------------------------------------------------------C
C   For the Model Params. that Depend on Covariates Enter the Equation C
C         {e.g. Pmean(1) =  PC(1)*R(2) }                               C
C----c-----------------------------------------------------------------C


C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine POPINIT(PmeanI,ICmeanI,PcovI,ICcovI, PCI)
C  Initial parameter values for population program parameters (ITS, MLEM)

        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Integer I,J
        Real*8 PmeanI(MaxNSP+MaxNDE), ICmeanI(MaxNDE)
        Real*8 PcovI(MaxNSP+MaxNDE,MaxNSP+MaxNDE), ICcovI(MaxNDE,MaxNDE)
        Real*8 PCI(MaxNCP)

CC
C----------------------------------------------------------------------C
C  Enter Initial Values for Population Means                           C
C          {  e.g. PmeanI(1) = 10.0    }                               C
C----c-----------------------------------------------------------------C


CC
C----------------------------------------------------------------------C
C   Enter Initial Values for Pop. Covariance Matrix (Lower Triang.)    C
C         {  e.g. PcovI(2,1) = 0.25    }                               C
C----c-----------------------------------------------------------------C


CC
C----------------------------------------------------------------------C
C   Enter Values for Covariate Model Parameters                        C
C         {  e.g. PCI(1) = 2.0    }                                    C
C----c-----------------------------------------------------------------C


C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine PRIOR(Pmean,Pcov,ICmean,ICcov)
C  Parameter mean and covariance values for MAP estimation (ID,NPD,STS)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Integer I,J
        Real*8 Pmean(MaxNSP+MaxNDE), ICmean(MaxNDE)
        Real*8 Pcov(MaxNSP+MaxNDE,MaxNSP+MaxNDE), ICcov(MaxNDE,MaxNDE)

CC
C----------------------------------------------------------------------C
C  Enter Nonzero Elements of Prior Mean Vector                         C
C          {  e.g. Pmean(1) = 10.0    }                                C
C----c-----------------------------------------------------------------C


CC
C----------------------------------------------------------------------C
C   Enter Nonzero Elements of Covariance Matrix (Lower Triang.)       C
C         {  e.g. Pcov(2,1) = 0.25    }                                C
C----c-----------------------------------------------------------------C


C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C

        Subroutine SPARAM(PS,P,IC)
        Implicit None

        Include 'globals.inc'

        Real*8 PS(MaxNSECP), P(MaxNSP+MaxNDE), IC(MaxNDE) 
	Real*8 mean_GAM,var_GAM
CC
C----------------------------------------------------------------------C
C       Enter Equations Defining Secondary Paramters                   C
C           {  e.g.  PS(1) = P(1)*P(2)   }                             C
C----c-----------------------------------------------------------------C

C  Calculate mean and variance 

	real*8 Een,KH
	real*8 Vmax,Km,Hen,Smax,SC50,TP,b

        Vmax = P(1)
        Km = P(2)
        Hen = P(3)
        Smax = P(4)
	SC50 = P(5)
	TP = P(6)
	b = P(7)

	KH = 29.5D0

	Een = (SC50*Hen)/(2D0*(1D0/b)*KH*Smax-Hen)

 	PS(1) = mean_GAM(2D0,b)
	PS(2) = var_GAM(2D0,b)
	PS(3) = Een
	PS(4) = KH*((Smax*Een)/(SC50+Een))*PS(1) 	




        
C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End
        
C######################################################################C

        Subroutine AMAT(A)
        Implicit None

        Include 'globals.inc'
        Include 'model.inc'

        Integer I,J
        Real*8 A(MaxNDE,MaxNDE)

        DO I=1,Ndeqs
           Do J=1,Ndeqs
              A(I,J)=0.0D0
           End Do
        End Do

CC
C----------------------------------------------------------------------C
C    Enter non zero elements of state matrix  {e.g.  A(1,1) = -P(1) }  C
C----c-----------------------------------------------------------------C



C----------------------------------------------------------------------C
C----------------------------------------------------------------------C
C
        Return
        End

C######################################################################C
